/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLprocessCreateLoader.java,v 1.2 2008-01-24 02:28:18 apnan Exp $
 * 
 * Contributors: IBM - Initial API and implementation
 ******************************************************************************/
package org.eclipse.hyades.loaders.hierarchy;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.models.hierarchy.HierarchyFactory;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.util.HierarchyResourceSetImpl;
import org.eclipse.hyades.models.hierarchy.util.SaveUtil;
import org.eclipse.hyades.models.hierarchy.util.StringUtil;
public class XMLprocessCreateLoader extends IgnoredXMLFragmentLoader {
	//~ Instance fields
	// ----------------------------------------------------------------------------
	protected String application_executable;
	protected String pName;
	protected String nodeIdRef;
	protected String processId;
	protected TRCProcessProxy process;
	protected int pid;
	//~ Methods
	// ------------------------------------------------------------------------------------
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.models.loader.XMLFragmentLoader#addAttribute(java.lang.String,
	 *      java.lang.String)
	 */
	public void addAttribute(String name, String value) {
		if (value == null || value.trim().length() == 0) 
			return;
		
		if (name.equals("processId")) {
			processId = value;
		} else if (name.equals("nodeIdRef")) {
			nodeIdRef = value;
		} else if (name.equals("name")) {
			pName = value;
		} else if (name.equals("pid")) {
			pid = Integer.parseInt(value);
		} else if (name.equals("application_executable")) {
			application_executable = value;
		}
	}

	protected TRCNode getNodeById(TRCMonitor monitor) {
		if (nodeIdRef == null) return null;
		
		TRCNode node = null;
		
		Iterator it = monitor.getNodes().iterator();
		while (it.hasNext()) {
			TRCNode n = (TRCNode) it.next();
			if (n.eIsProxy()) continue;
			
			if (nodeIdRef.equals(n.getRuntimeId())) {
				node = n;
				break;
			}
		}
		
		return node;
	}

	protected TRCProcessProxy getProcess (TRCNode node, int id) {
		TRCProcessProxy proc = null;

	    EList processList = node.getProcessProxies();
	    for (int i = 0; i<processList.size(); i++) {
	        TRCProcessProxy p = (TRCProcessProxy) processList.get(i);
	        if (p.getPid() == id) {
	        	proc = p;
	        	break;
	        }
	    }
		
	    return proc;
	}
	
	protected TRCNode getNodeByPid(TRCMonitor monitor) {
		if (processId == null) return null;
		
		TRCNode node = null;

		Iterator itn = monitor.getNodes().iterator();
		while (itn.hasNext()) {
			TRCNode n = (TRCNode) itn.next();
			if (n.eIsProxy()) continue;

			TRCProcessProxy proc = getProcess(n, pid);
			if (proc != null) {
				node = n;
				break;
			}
		}
		
		return node;
	}

	protected TRCNode getNode(TRCMonitor monitor) {
		return (nodeIdRef != null) ? getNodeById(monitor) : getNodeByPid(monitor);
	}

	public void addYourselfInContext() {
		TRCMonitor monitor = context.getMonitor();
		if (monitor == null) {
			return;
		}

		
		TRCNode node = context.getNode(); 
		if(node==null)	
		{
			node = getNode(monitor);
			if (node == null) {
				return;
			}
			else
				context.setNode(node);
		}
		
		process = LoadersUtils.locateProcessFromNode(node, pid, processId);
		if (process == null) {
			String monitorFolder = LoadersUtils.getMonitorFolder(monitor);
			Resource procDoc = null;
			if (monitorFolder != null) {
				//create the process
				StringBuffer buffer = (new StringBuffer(monitor.getName())).append("_").append(node.getName()).append("_").append(pid).append("_").append(processId);
				String pName = buffer.toString();
//				pName = StringUtil.change(pName, " ", "");
				pName = StringUtil.change(pName, "\"", "");
				String pFileName = monitorFolder + pName;
				URI uri = SaveUtil.createURI(pFileName + ".trcpxmi#").trimFragment();
				procDoc = Resource.Factory.Registry.INSTANCE.getFactory(uri).createResource(uri);
				procDoc.setModified(true);
				HierarchyResourceSetImpl.getInstance().getResources().add(procDoc);
				//*** adding support for multiple files
				//                SaveUtil.addDocument(procDoc);
			}
			process = HierarchyFactory.eINSTANCE.createTRCProcessProxy();
			process.setNode(node);
			if (procDoc != null) {
				procDoc.getContents().add(process);
			}
		}

		process.setPid(pid);
		if (processId != null) {
			process.setRuntimeId(processId);
		} else if (process.getRuntimeId() == null) {
			process.setRuntimeId("0");
		}
		
		setProcessArguments(process);
		
		if(process.getName()==null || process.getName().equals(Constants.UNKNOWN))
			process.setName(pName);
		
		context.setProcessProxy(process);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#initialize(org.eclipse.hyades.loaders.util.HierarchyContext,
	 *      java.lang.String)
	 */
	public void initialize(HierarchyContext context, String name) {
		super.initialize(context, name);
		processId = "0";
		nodeIdRef = "0";
		application_executable = "";
		pName = Constants.UNKNOWN;
		process = null;
		pid = -1;
	}
	protected void setProcessArguments(TRCProcessProxy process) {
		if (application_executable == null) {
			return;
		}
		String params = "";
		String vmparam = "";
		String classpath = "";
		int java_index = application_executable.indexOf("java.exe");
		if (java_index < 0) {
			java_index = application_executable.indexOf("javaw.exe");
			if (java_index > 0) {
				java_index += "javaw.exe".length();
			}
		} else {
			java_index += "java.exe".length();
		}
		if (java_index > 0) {
			String exeName = application_executable.substring(java_index).trim();
			//            if (!exeName.startsWith("-X")) { //log agents
			//                exeName = "-X " + exeName;
			//            }
			int idx = exeName.indexOf(" ");
			if (!exeName.startsWith("-")) {
				if ((idx != -1) && (idx < (exeName.length() + 1))) {
					exeName = exeName.substring(idx + 1);
				}
			}
			while (exeName.startsWith("-")) {
				if (exeName.startsWith("-cp") || exeName.startsWith("-classpath")) {
					idx = exeName.indexOf(" ");
					if (idx != -1) {
						exeName = exeName.substring(idx).trim();
					}
					idx = exeName.indexOf(" ");
					if (idx != -1) {
						classpath = exeName.substring(0, idx);
						exeName = exeName.substring(idx).trim();
					}
				} else {
					idx = exeName.indexOf(" ");
					if (idx != -1) {
						vmparam += (exeName.substring(0, idx) + " ");
						exeName = exeName.substring(idx).trim();
					}
				}
			}
			idx = exeName.indexOf(" ");
			if (idx != -1) {
				if (pName.equals(Constants.UNKNOWN)) {
					pName = exeName.substring(0, idx);
				}
				params = exeName.substring(idx).trim();
			} else {
				if (pName.equals(Constants.UNKNOWN) && exeName.trim().length() > 0)
					pName = exeName;
			}
		}
		process.setClasspath(classpath);
		process.setParameters(params);
		process.setVmArguments(vmparam);
//		process.setLaunchMode(1);
	}
}