/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLmonContendedEnterLoader.java,v 1.16 2010/09/08 17:42:01 jwest Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * Intel - loader is extended for contention analysis support  
 * 
 * $Id: XMLmonContendedEnterLoader.java,v 1.16 2010/09/08 17:42:01 jwest Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import java.util.List;

import org.eclipse.hyades.loaders.hierarchy.Constants;
import org.eclipse.hyades.loaders.trace.ThreadDeadlockDetector.DeadlockEntry;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.trace.TRCHeapObject;
import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TRCThreadDeadLockEvent;
import org.eclipse.hyades.models.trace.TRCThreadWaitingForLockEvent;
import org.eclipse.hyades.models.trace.TraceFactory;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCFullHeapObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCHeapObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCThreadImpl;


public class XMLmonContendedEnterLoader extends TraceXMLThreadEventsLoader {


	// ~ Static fields/initializers
	// -----------------------------------------------------------------
//	private static final String THREAD_OWNER = "threadOwner";
	
	// ~ Instance fields
	// ----------------------------------------------------------------------------
	private int threadOwner;

	// ~ Methods
	// ------------------------------------------------------------------------------------
	public void addAttribute(String name, String value) {
		
		switch (LoadersUtils.getHashCode(name)) {
		case TraceConstants.THREAD_OWNER_int:
			threadOwner = Integer.parseInt(value);
			break;
		default:
			super.addAttribute(name, value);
			break;
		}
	}

	public void addYourselfInContext() {
		if (context.isFilterOn()) {
			if (context.getFilterEngine().isFiltered( TracePackage.eINSTANCE.getTRCThread_Name(), LoadersUtils.getLookUpKey(threadIdRef))) {
				return;
			}
			if (context.getFilterEngine().isFiltered(
					TracePackage.eINSTANCE.getTRCThread_Name(),
					LoadersUtils.getLookUpKey(threadOwner))) {
				return;
			}
		}
		theProcess = getProcess();
		theThread = getThreadByIdRef(theProcess);
		dispatchProcessMode(ProcessSteps.ALL);
		TRCThread lockingThread = null;

		// Create the TRCThreadWaitingForLockEvent
		
		TRCThreadWaitingForLockEvent threadWaitingForLockEvent = TraceFactory.eINSTANCE.createTRCThreadWaitingForLockEvent();
		threadWaitingForLockEvent.setLockedObject(theObject);
		threadWaitingForLockEvent.setThread(theThread);
		
		threadWaitingForLockEvent.setTime(createDeltaTime());// time);
		if (threadOwner > 0) {
			lockingThread = (TRCThread) LookupServiceExtensions.getInstance() .locate(context, TRCThreadImpl.class, threadOwner);
			threadWaitingForLockEvent.setLockingThread(lockingThread);
			getThreadEventsContext().getMonitor2OwnerMap().put(theObject, lockingThread);
		}
		addAnnotationsIfRequired(threadWaitingForLockEvent);

		theThread.getThreadEvents().add(threadWaitingForLockEvent);
		
		// Search for deadlock
		
		ThreadDeadlockDetector detector = getThreadEventsContext().getThreadDeadlockDetector();
		
		detector.handleMonContendedEnter(threadIdRef, objIdRef, threadOwner);
		
		checkForDeadlockedThread(context, this, detector);
		
	}
	
	public static void checkForDeadlockedThread(HierarchyContext context, TraceXMLFragmentLoader loader, ThreadDeadlockDetector detector) {
		
		List<DeadlockEntry> deadLockedThreads = detector.checkWaitingDeadlocks();

		if(deadLockedThreads != null && deadLockedThreads.size() > 0) {

			for(DeadlockEntry c  : deadLockedThreads) {


				TRCThread otherThread = (TRCThread) LookupServiceExtensions.getInstance().locate(context, TRCThreadImpl.class, c._objectLockedByWhichThread);
				
				TRCThreadDeadLockEvent e = TraceFactory.eINSTANCE.createTRCThreadDeadLockEvent();
				e.setLockedObject((TRCObject)LookupServiceExtensions.getInstance().locate(context, TRCObjectImpl.class, c._lockedObject));

				e.setLockingThread(otherThread);
			
				e.setNextDeadLockEvent(null);  // (not used anywhere else)
				e.setThread((TRCThread) LookupServiceExtensions.getInstance().locate(context, TRCThreadImpl.class, c._lockedThread));
				e.setTime(loader.createDeltaTime());
				
				e.getThread().getThreadEvents().add(e);
				
			}
			
		}
		
	}

	public void initialize(HierarchyContext context, String name) {
		loadToModel = context.isLoadToModel();
		super.initialize(context, name);
		threadOwner = 0;
		annotations.clear();
		currentAnnotationValueEntryActive = false;

	}

	public void cleanUp() {
		super.cleanUp();
		threadOwner = 0;
	}

	protected void processHF(int step) {
		super.processHF(step);
		Class clazz = (TraceUtils.isBooleanOptionEnabled(context,
				Constants.MULTIPLE_HEAP_DUMPS) ? TRCFullHeapObjectImpl.class
				: TRCHeapObjectImpl.class);
		if ((theObject == null) || !(theObject instanceof TRCHeapObject)) {
			theObject = (TRCHeapObject) LookupServiceExtensions.getInstance()
					.locate(context, clazz, objIdRef);
		}
	}
	
}