/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLobjFreeLoader.java,v 1.7 2006-02-08 19:56:55 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLobjFreeLoader.java,v 1.7 2006-02-08 19:56:55 slavescu Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import org.eclipse.hyades.loaders.internal.binary.v1.BinaryTraceConstants;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TRCHeapObject;
import org.eclipse.hyades.models.trace.TRCPackage;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCFullTraceObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCHeapObjectImpl;


/** @author slavescu
 *
 */
public class XMLobjFreeLoader extends TraceXMLFragmentLoader {
    //~ Methods ------------------------------------------------------------------------------------

    public void addAttribute(String name, String value) {
        super.addAttribute(name, value);
    }
    
    public void addAttribute(int con, long value) {
    	switch(con) {

    	case BinaryTraceConstants.OBJ_AGE:
    	case BinaryTraceConstants.TRANS_THREAD_ID_REF:
    	case BinaryTraceConstants.TRANS_OBJ_ID_REF:
    	case BinaryTraceConstants.SEQUENCE_COUNTER:
    	case BinaryTraceConstants.STACK_DEPTH:
    		// ignore
    		break;
   		default:
   			// OBJ_ID_REF: 
   			super.addAttribute(con, value);
    		break;
    	}
    }
    
    public void addAttribute(int con, String value) {
    	switch(con) {   
    	case BinaryTraceConstants.TRACE_ID_REF:
    	case BinaryTraceConstants.COLLATION_VALUE:
    		// ignore
    		break;
    	default:
    		// TIME
    		super.addAttribute(con, value);
    		break;
    		
    	}
    }
    
    public void addAttribute(int con, int value) {
    	switch(con) {
    	default:
    		// THREAD_ID_REF
    		super.addAttribute(con, value);
    		break;
    	}
    }

    public void addYourselfInContext() {
		if(context.isFilterOn())
		{
			if(context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCClass_Name(),LoadersUtils.getLookUpKey(classIdRef)))
			{
				return;
			}
		}
        if (!loadToModel) {
            TraceUtils.removeVirtualObject(context, objIdRef);

            return;
        }

        theProcess = getProcess();

        dispatchProcessMode(ProcessSteps.ALL);
        TraceUtils.removeVirtualObject(context, objIdRef);
    }

    public void initialize(HierarchyContext context, String name) {
        loadToModel = context.isLoadToModel();
        super.initialize(context, name);
    }
	
    protected void processENI(int step) {
        if (virtualObject == null && theObject==null) {
            useVirtualObject();
        }
	}
    /**
     *
     */
    protected void processEF(int step) {
        super.processEF(step);

		if(theObject!=null)
		{
            theObject = (TRCFullTraceObject) LookupServiceExtensions.getInstance().locate(context, TRCFullTraceObjectImpl.class, objIdRef);
	
	        if ((theObject == null) || (((TRCFullTraceObject) theObject).getCollectTime() != 0.0)) {
	            return;
	        }
	
	        ((TRCFullTraceObject) theObject).setCollectTime(createDeltaTime());
		}
		else
		{
			theObject = (TRCFullTraceObject) LookupServiceExtensions.getInstance().locate(context, TRCFullTraceObjectImpl.class, objIdRef);
	
			if ((theObject == null) || (((TRCFullTraceObject) theObject).getCollectTime() != 0.0)) {
				return;
			}
	
			((TRCFullTraceObject) theObject).setCollectTime(createDeltaTime());
			
			theClass = theObject.getIsA();

			updateClassStatistics(theObject.getSize());

		}
    }

    /**
     *
     */
    protected void processES(int step) {
        super.processES(step);

        if (virtualObject == null && theObject==null) {
            useVirtualObject();
        }
    }

    /**
     *
     */
    protected void processHF(int step) {
        super.processHF(step);
        
        if(theObject!=null)
        {
			return;
        }
			
        theObject = (TRCHeapObject) LookupServiceExtensions.getInstance().locate(context, TRCHeapObjectImpl.class, objIdRef);

        if (theObject == null) {
            return;
        }
		theClass = theObject.getIsA();
	
		updateClassStatistics(theObject.getSize());

    }

    /**
     *
     */
    protected void processHS(int step) {
        super.processHS(step);

        if (virtualObject == null && theObject==null) {
            useVirtualObject();
        }
    }

    private void updateClassStatistics(int s) {

    	theClass.incrementCollectedInstances(1);
//        theClass.setCollectedInstances(theClass.getCollectedInstances() + 1);

    	theClass.incrementCollectedSize(s);
//        theClass.setCollectedSize(theClass.getCollectedSize() + s);

        TRCPackage thePackage = theClass.getPackage();

        thePackage.incrementCollectedInstances(1);
//        thePackage.setCollectedInstances(thePackage.getCollectedInstances() + 1);
        
        
        
        thePackage.incrementCollectedSize(s);
//        thePackage.setCollectedSize(thePackage.getCollectedSize() + s);
        
        
        theProcess.incrementCollectedInstances(1);
//        theProcess.setCollectedInstances(theProcess.getCollectedInstances() + 1);
        
        theProcess.incrementCollectedSize(s);
//        theProcess.setCollectedSize(theProcess.getCollectedSize() + s);

        //		updateSuperPackagesStatistics(thePackage,size);
    }

    private void useVirtualObject() {
        virtualObject = (VirtualObjectInfo) LookupServiceExtensions.getInstance().locate(context, VirtualObjectInfo.class, objIdRef);

        if (virtualObject != null) {
            theClass = virtualObject.myClass;

            if (theClass != null) {
                updateClassStatistics(virtualObject.size);
            }

            TraceUtils.removeVirtualObject(context, objIdRef);
        }
    }
}
