/**********************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ArtifactEditorExtension.java,v 1.9 2008-04-17 02:21:06 paules Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.test.ui.internal.editor.extension;

import java.util.Vector;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.hyades.models.common.configuration.CFGArtifact;
import org.eclipse.hyades.models.common.facades.behavioral.ITestSuite;
import org.eclipse.hyades.test.ui.editor.form.util.WidgetFactory;
import org.eclipse.hyades.test.ui.internal.editor.form.ArtifactForm;
import org.eclipse.hyades.test.ui.internal.editor.form.ConfigurableObjectOverviewForm;
import org.eclipse.hyades.test.ui.internal.editor.form.TestAssetsForm;
import org.eclipse.hyades.test.ui.internal.resources.UiPluginResourceBundle;
import org.eclipse.jface.viewers.IStructuredSelection;

/**
 * Hyades configurable object editor extension that is suitable for all 
 * {@link org.eclipse.hyades.models.common.configuration.CFGArtifact} objects.
 * <p/>
 * 
 * 
 * @author  Bianca Xue Jiang
 * @author  Paul E. Slauenwhite
 * @version April 16, 2008
 * @since   February 1, 2005
 */
public class ArtifactEditorExtension extends ConfigurableObjectEditorExtension 
{
	public static final String EXTENSION_ID = "org.eclipse.hyades.test.ui.editor.extension.Artifact"; //$NON-NLS-1$
	public static final String EDITOR_OBJECT_TYPE = CFGArtifact.class.getName(); //"org.eclipse.hyades.models.common.configuration.CFGArtifact"
	
	private static final int PAGE_TEST_ASSETS = 1;	
	private TestAssetsForm testAssetsForm;
	
	protected IConfigurationElement[] collectPropertyGroupExtensions() 
	{
		IConfigurationElement[] extensions = super.collectPropertyGroupExtensions();		
		Vector registeredExts = new Vector();		 
		{
			for( int i = 0; i < extensions.length; i++ )
			{
			   if( extensions[i].getAttribute("appliesTo").equals(EDITOR_OBJECT_TYPE)) //$NON-NLS-1$
			   {
				   registeredExts.add( extensions[i] );
			   }
			}
		}	
	
		if( extensions.length == registeredExts.size() )
			return extensions;	
	
		IConfigurationElement[] registeredExtensions = new IConfigurationElement[registeredExts.size()];
		for( int i = 0; i < registeredExts.size(); i++ )
		{
			registeredExtensions[i] = (IConfigurationElement)registeredExts.get(i);
		}
		return registeredExtensions;
	}
	
	/**
	 * @see org.eclipse.hyades.ui.util.IDisposable#dispose()
	 */
	public void dispose()
	{
		if(testAssetsForm != null)
		{
		    testAssetsForm.dispose();
		    testAssetsForm = null;
		}	
		super.dispose();
	}

	/**
	 * @see org.eclipse.hyades.ui.editor.IEditorExtension#createPages()
	 */
	protected ConfigurableObjectOverviewForm createOverviewPage(WidgetFactory widgetFactory)
	{
	    ArtifactForm overviewForm = new ArtifactForm(this, widgetFactory); 
	    overviewForm.setTestAssetsPageIndex(PAGE_TEST_ASSETS);
		return overviewForm;
	}
	
	protected void createOtherStaticPages(WidgetFactory widgetFactory)
	{
		// insert a new form of its own which is after the overview form and before any pages added by extensions.
		testAssetsForm = new TestAssetsForm(this, widgetFactory);
		getHyadesEditorPart().addPage(testAssetsForm.createControl());
		getHyadesEditorPart().setPageText(PAGE_TEST_ASSETS, UiPluginResourceBundle.W_TEST_ASSETS);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.test.ui.internal.editor.IExtensibleEditorExtension#activateExtensionPage(org.eclipse.core.runtime.IConfigurationElement)
	 */
	public int addExtensionPage(IConfigurationElement extensionElement) 
	{
		// if this extension element is not intended for location editor, return -1.
		if( !extensionElement.getAttribute("appliesTo").equals(EDITOR_OBJECT_TYPE)) //$NON-NLS-1$
			return -1;
	    
		return super.addExtensionPage(extensionElement);
	}
	
	/**
	 * @see org.eclipse.hyades.ui.util.IRefreshable#refreshContent(java.lang.Object)
	 */
	public void refreshContent(Object data)
	{
		super.refreshContent(data);		
		testAssetsForm.load();
	}
	
	/**
	 * @see org.eclipse.hyades.ui.editor.IEditorExtension#pageActivated(int)
	 */
	public boolean pageActivated(int index)
	{
		if(index == PAGE_TEST_ASSETS)
			return testAssetsForm.activated();
		else
			return super.pageActivated(index);
	}

	/**
	 * @see org.eclipse.hyades.ui.editor.IEditorExtension#getSelection()
	 */
	public IStructuredSelection getSelection()
	{
		if(PAGE_TEST_ASSETS == getHyadesEditorPart().getActivePage())
			return (IStructuredSelection)testAssetsForm.getSelection();
		else
			return super.getSelection();
	}	
	
	/* (non-Javadoc)
	 * @see org.eclipse.hyades.ui.editor.EditorExtension#setSelection(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void setSelection(IStructuredSelection structuredSelection){

		if(structuredSelection.getFirstElement() instanceof ITestSuite){

			if(PAGE_TEST_ASSETS < getHyadesEditorPart().getPageCount()){

				getHyadesEditorPart().setActivePage(PAGE_TEST_ASSETS);
				testAssetsForm.selectReveal(structuredSelection);
			}
		}
		else{
			super.setSelection(structuredSelection);
		}
	}
}
