/*******************************************************************************
 * Copyright (c) 2005, 2006 Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Guru Nagarajan, Intel - Initial API and Implementation (TPTP 4.0)
 * 	  
 * $Id$
 *******************************************************************************/
package org.eclipse.tptp.platform.execution.client.core.internal.commands;

import java.util.Enumeration;

import org.eclipse.tptp.platform.execution.client.core.internal.ConnectionImpl;
import org.eclipse.tptp.platform.execution.client.core.internal.ControlMessage;
import org.eclipse.tptp.platform.execution.util.*;
import org.eclipse.tptp.platform.execution.util.internal.CommandElement;
import org.eclipse.tptp.platform.execution.util.internal.Constants;
import org.eclipse.tptp.platform.execution.util.internal.TPTPMessageUtil;
import org.eclipse.tptp.platform.execution.util.internal.TPTPString;
import org.eclipse.tptp.platform.execution.util.internal.TPTPXMLParse;

public class DetachFromAgentCommand extends CommandElement {

		protected String 	_agentName		= null;
		protected int	 	_agentID		= -1;
		protected String	_commandName 	= null;
				
		public DetachFromAgentCommand(String commandName, String agentname)
		{
			this._agentName = agentname;
			this._commandName = commandName;
		}
		public DetachFromAgentCommand(String commandName, int agentID)
		{
			this._agentID 	= agentID;
			this._commandName = commandName;
		}
		

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getSize()
		 */
		public int getSize() 
		{
			return 0;
		}
		public long getContext() 
		{
			return _context;
		}
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setContextID(java.lang.String)
		 */
		public void setContext(long contextID) {
			_context = contextID;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getSrcID()
		 */
		public long getSource() {
			return _src;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setSrcID(java.lang.String)
		 */
		public void setSource(long srcID) {
			_src = srcID;
		}
		//No TAG for TPTP 4.0
		public long getTag() {
			return _tag;
		}


		public long getDestination() {
			return _dest;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getInterfaceID()
		 */
		public String getInterfaceID() {
			return _iid;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setInterfaceID(java.lang.String)
		 */
		public void setInterfaceID(String interfaceID) {
			_iid = interfaceID;
		}
		
		public void setDestination(long dest) {
			_dest=dest;
		}
	 			
			
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#buildCommand(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
		 */
		public String buildCommand() 
		{
			StringBuffer sbuf = new StringBuffer();
			sbuf.append("<");
			sbuf.append(this._commandName + " iid=\"");
			sbuf.append(Constants.AGNT_MGR_IID+"\"");
			sbuf.append(">");
			if(this._commandName != null)
			{
				if(this._agentID > 0)
				{
					sbuf.append("<agentID>"+this._agentID+"</agentID>");
				}
				if(this._agentName != null)
				{
					sbuf.append("<agentName>"+this._agentName+"</agentName>");
				}
			}
			sbuf.append("</"+this._commandName+">");
		
			return sbuf.toString();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.IProcess#setExecutable(java.lang.String)
		 */
		/**
		* Read the command from the input buffer.
		* @return int
		* @param buffer byte[]
		* @param offset int
		*/
		public int readFromBuffer(byte[] buffer, int offset)
		{
			int current=offset;
			String retStr = null;
			current=TPTPMessageUtil.readTPTPStringFromBuffer(buffer, current, new TPTPString(retStr));
			//With the retStr value received parse the output for the specific elements
			TPTPXMLParse ParseObj = new TPTPXMLParse();
			ParseObj.setParser(retStr);
			Enumeration enenumerum = ParseObj.getHashTable().elements();
			
			if(ParseObj.getHashTable().containsKey("AgentName"))
			{
				_agentName = (String)((ParseObj.getHashTable()).get("AgentName")) ;
			}	
			return current;
		}

		/**
		* Write the XML Command to the input buffer.
		* @return int
		* @param buffer byte[]
		* @param offset int
		*/
		public int writeToBuffer(byte[] buffer, int offset)
		{
			int current=offset;
			current=TPTPMessageUtil.writeTPTPStringToBuffer(buffer, current, new TPTPString(this.buildCommand()));
			return current;
		}

}
