/*******************************************************************************
 * Copyright (c) 2005, 2006 Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Guru Nagarajan, Intel - Initial API and Implementation 
 * $Id$
 *******************************************************************************//*
 * Created on May 2, 2005
 *
 */
package org.eclipse.tptp.platform.execution.client.core.internal.commands;

/**
 * @author gnagaraj
 *
 */

import java.util.Enumeration;

import org.eclipse.tptp.platform.execution.util.internal.CommandElement;
import org.eclipse.tptp.platform.execution.util.internal.Constants;
import org.eclipse.tptp.platform.execution.util.internal.TPTPMessageUtil;
import org.eclipse.tptp.platform.execution.util.internal.TPTPString;
import org.eclipse.tptp.platform.execution.util.internal.TPTPXMLParse;

/**
 * @author gnagaraj
 *
 */
public class QueryRunningAgentsCommand extends CommandElement {

		protected String _commandData = null;
		protected String[] _agentInterface = null;
		protected String  _agentName = null;
		protected long	  _processID = -1;
				
		public QueryRunningAgentsCommand()
		{
			this._commandData = new String();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getSize()
		 */
		public int getSize() 
		{
			return 0;
		}
		public long getContext() 
		{
			return _context;
		}
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setContextID(java.lang.String)
		 */
		public void setContext(long contextID) {
			_context = contextID;
		}
		public long getProcessId() 
		{
			return _processID;
		}
		public void setProcessId(long processID) {
			_processID = processID;
		}
		public String getAgentName() 
		{
			return _agentName;
		}
		public void setAgentName(String agentName) {
			_agentName = agentName;
		}
		 /* @see org.eclipse.tptp.platform.execution.core.impl.commands.QueryRunningAgentsCommand#setCommand(java.lang.String)
		 */
		public void setCommandData(String command) {
			this._commandData = new String(command);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.impl.commands.QueryRunningAgentsCommand#getCommandData()
		 */
		public String getCommandData() {
			return this._commandData;
		}
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getSrcID()
		 */
		public long getSource() {
			return _src;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setSrcID(java.lang.String)
		 */
		public void setSource(long srcID) {
			_src = srcID;
		}
		//No TAG for TPTP 4.0
		public long getTag() {
			return _tag;
		}

		public long getDestination() {
			return _dest;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getInterfaceID()
		 */
		public String getInterfaceID() {
			return _iid;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setInterfaceID(java.lang.String)
		 */
		public void setInterfaceID(String interfaceID) {
			_iid = interfaceID;
		}
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#getInterfaceID()
		 */
		public String[] getAgentInterfaceID() {
			return this._agentInterface;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#setInterfaceID(java.lang.String)
		 */
		public void setAgentInterfaceID(int numCount, String[] agentType) 
		{
			if (numCount == 0) return; 
			this._agentInterface = new String[numCount];
			
			for(int i = 0; i < agentType.length; i++)
			{
				this._agentInterface[i] = agentType[i];	
			}
		}
		
		public void setDestination(long dest) {
			_dest=dest;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.ICommandElement#buildCommand(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
		 */
		public String buildCommand() 
		{
			StringBuffer sbuf = new StringBuffer();
			sbuf.append("<");
			sbuf.append(Constants.queryRunningAgents_Cmd + " iid=\"");
			sbuf.append(Constants.AGNT_MGR_IID+"\"");
			sbuf.append(">");
			if(this._agentInterface != null)
			{
			  for(int i = 0; i < this._agentInterface.length; i++)
			  {
				sbuf.append("<interface>"+this._agentInterface[i]+"</interface>");
			  }
			}
			if(this._agentName != null)
			{
			    sbuf.append("<agentName>"+this._agentName+"</agentName>");
			}
			if(this._processID != -1)
			{
			    sbuf.append("<processID>"+this._processID+"</processID>");
			}
			sbuf.append("</"+Constants.queryRunningAgents_Cmd+">"); 
			
			return sbuf.toString();
		}
		/* (non-Javadoc)
		 * @see org.eclipse.tptp.platform.execution.core.IProcess#setExecutable(java.lang.String)
		 */
				/**
		* Read the command from the input buffer.
		* @return int
		* @param buffer byte[]
		* @param offset int
		*/
		public int readFromBuffer(byte[] buffer, int offset)
		{
			int current=offset;
			String retStr = null;
			current=TPTPMessageUtil.readTPTPStringFromBuffer(buffer, current, new TPTPString(retStr));
			//With the retStr value received parse the output for the specific elements
			TPTPXMLParse ParseObj = new TPTPXMLParse();
			ParseObj.setParser(retStr);
			Enumeration enumer = ParseObj.getHashTable().elements();
			
			//Build the AgentMetadata details reading from the XML Response	
			return current;	
		
		}


		/**
		* Write the XML Command to the input buffer.
		* @return int
		* @param buffer byte[]
		* @param offset int
		*/
		public int writeToBuffer(byte[] buffer, int offset)
		{
			int current=offset;
			current=TPTPMessageUtil.writeTPTPStringToBuffer(buffer, current, new TPTPString(buildCommand()));
			return current;
		}

}
